@@ -1,5 +1,5 @@
 /* BFD back-end for ALPHA Extended-Coff files.
-   Copyright (C) 1993-2025 Free Software Foundation, Inc.
+   Copyright (C) 1993-2026 Free Software Foundation, Inc.
    Modified from coff-mips.c by Steve Chamberlain <sac@cygnus.com> and
    Ian Lance Taylor <ian@cygnus.com>.
 
@@ -861,7 +861,7 @@ alpha_ecoff_get_relocated_section_contents (bfd *abfd,
     {
       arelent *rel;
       bfd_reloc_status_type r;
-      char *err;
+      char *err = NULL;
       unsigned int r_type;
 
       rel = *relp;
@@ -1154,43 +1154,10 @@ alpha_ecoff_get_relocated_section_contents (bfd *abfd,
 
       if (r != bfd_reloc_ok)
 	{
-	  switch (r)
-	    {
-	    case bfd_reloc_undefined:
-	      (*link_info->callbacks->undefined_symbol)
-		(link_info, bfd_asymbol_name (*rel->sym_ptr_ptr),
-		 input_bfd, input_section, rel->address, true);
-	      break;
-	    case bfd_reloc_dangerous:
-	      (*link_info->callbacks->reloc_dangerous)
-		(link_info, err, input_bfd, input_section, rel->address);
-	      break;
-	    case bfd_reloc_overflow:
-	      (*link_info->callbacks->reloc_overflow)
-		(link_info, NULL, bfd_asymbol_name (*rel->sym_ptr_ptr),
-		 rel->howto->name, rel->addend, input_bfd,
-		 input_section, rel->address);
-	      break;
-	    case bfd_reloc_outofrange:
-	      (*link_info->callbacks->einfo)
-		/* xgettext:c-format */
-		(_("%X%P: %pB(%pA): relocation \"%pR\" goes out of range\n"),
-		 input_bfd, input_section, rel);
-	      goto error_return;
-	    case bfd_reloc_notsupported:
-	      (*link_info->callbacks->einfo)
-		/* xgettext:c-format */
-		(_("%X%P: %pB(%pA): relocation \"%pR\" is not supported\n"),
-		 input_bfd, input_section, rel);
-	      goto error_return;
-	    default:
-	      (*link_info->callbacks->einfo)
-		/* xgettext:c-format */
-		(_("%X%P: %pB(%pA): relocation \"%pR\""
-		   " returns an unrecognized value %x\n"),
-		 input_bfd, input_section, rel, r);
-	      break;
-	    }
+	  _bfd_link_reloc_status_error (abfd, link_info, input_section,
+					rel, err, r);
+	  if (r == bfd_reloc_outofrange || r == bfd_reloc_notsupported)
+	    goto error_return;
 	}
     }
 
@@ -2235,8 +2202,10 @@ alpha_ecoff_openr_next_archived_file (bfd *archive, bfd *last_file)
 {
   ufile_ptr filestart;
 
+  BFD_ASSERT (!bfd_is_fake_archive (archive));
+
   if (last_file == NULL)
-    filestart = bfd_ardata (archive)->first_file_filepos;
+    filestart = bfd_ardata (archive)->first_file.file_offset;
   else
     {
       struct areltdata *t;
@@ -2252,9 +2221,9 @@ alpha_ecoff_openr_next_archived_file (bfd *archive, bfd *last_file)
       /* Pad to an even boundary...
 	 Note that last_file->origin can be odd in the case of
 	 BSD-4.4-style element with a long odd size.  */
-      filestart = last_file->proxy_origin + size;
+      filestart = last_file->proxy_handle.file_offset + size;
       filestart += filestart % 2;
-      if (filestart < last_file->proxy_origin)
+      if (filestart < last_file->proxy_handle.file_offset)
 	{
 	  /* Prevent looping.  See PR19256.  */
 	  bfd_set_error (bfd_error_malformed_archive);
@@ -2273,7 +2242,10 @@ alpha_ecoff_get_elt_at_index (bfd *abfd, symindex sym_index)
   carsym *entry;
 
   entry = bfd_ardata (abfd)->symdefs + sym_index;
-  return alpha_ecoff_get_elt_at_filepos (abfd, entry->file_offset, NULL);
+  if (bfd_ardata (abfd)->symdef_use_bfd)
+    return entry->u.abfd;
+  else
+    return alpha_ecoff_get_elt_at_filepos (abfd, entry->u.file_offset, NULL);
 }
 
 static void
@@ -2456,7 +2428,7 @@ static const struct ecoff_backend_data alpha_ecoff_backend_data =
 #define _bfd_ecoff_bfd_link_check_relocs    _bfd_generic_link_check_relocs
 
 /* Installing internal relocations in a section is also generic.  */
-#define _bfd_ecoff_set_reloc _bfd_generic_set_reloc
+#define _bfd_ecoff_finalize_section_relocs _bfd_generic_finalize_section_relocs
 
 const bfd_target alpha_ecoff_le_vec =
 {
